# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import unicodedata
import six

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String, Multiple


class CovidResultsBase(Splitted):

    SEPARATOR = ","


class CovidResult(object):
    FIELDS = [
        String("well"),
        String("seq_no"),
        String("sample"),
        String("sample_id"),
        Multiple("results", (
            String("resu"),
        ), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class CovidResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: CovidResult
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf_data = unicodedata.normalize('NFKD', six.text_type(data)).encode("ascii", "ignore")
        if six.PY3:
            buf_data = buf_data.decode("ascii")
        buf = six.StringIO(buf_data[1:])
        reader = csv.reader(buf, delimiter=str(','))
        for line in reader:
            if not len(line):
                continue
            if len(line) > 5:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {
        "code": "livecycle",
        "label": "AG APPOLLON CSV"
    }

    def parse(self, data):
        obj = CovidResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.seq_no.strip() == "Seq no":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample_id,
                    rack="COVID-{}".format(m.well),
                    type="patient",
                )
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index] and result:
                        sample.results.add(
                            code=RESULTS_CODES[index].strip(),
                            value=result.replace(",", ".")
                        )
        return worklist
