# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
from six import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String


class ThermocoupleurBase(Splitted):
    SEPARATOR = ";"


class ThermocoupleurResult(object):
    FIELDS = [
        String("sample_id"),
        String("code"),
        String("result"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            setattr(self, field.name, line[i])


class ThermocoupleurMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: ThermocoupleurResult
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(';'))
        for line in reader:
            if not len(line):
                continue
            if len(line) == 3:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {
        "code": "thermocoupleur_csv",
        "label": "Thermocoupleur csv Générique"
    }

    def parse(self, data):
        obj = ThermocoupleurMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample = worklist.get_or_create(
                sample_id=m.sample_id,
                type="patient"
            )
            sample.results.add(
                code=m.code,
                value=m.result,
            )
        return worklist
